# commit_formats.py

from enum import Enum
from pydantic import BaseModel, Field


class CommitType(str, Enum):
    feat = "feat"
    fix = "fix"
    docs = "docs"
    style = "style"
    refactor = "refactor"
    perf = "perf"
    test = "test"
    build = "build"
    ci = "ci"
    chore = "chore"
    revert = "revert"


class ConventionalCommit(BaseModel):
    type: CommitType
    scope: str = Field(default="", max_length=50)
    description: str = Field(..., max_length=100)
    long_description: str = Field(default="", max_length=1000)


EMOJI_MAP = {
    "feat": "✨",
    "fix": "🐛",
    "docs": "📚",
    "style": "💎",
    "refactor": "📦",
    "perf": "🚀",
    "test": "🚨",
    "build": "🛠️",
    "ci": "⚙️",
    "chore": "♻️",
    "revert": "⏪"
}


def truncate_to_sentence(text: str, max_length: int) -> str:
    """
    Truncate text to the last complete sentence that fits within max_length.
    """
    if not text or len(text) <= max_length:
        return text

    # Find the last sentence boundary within the limit
    sentence_endings = ['. ', '! ', '? ']
    last_boundary = -1

    for i in range(min(len(text), max_length), -1, -1):
        if any(text[i - 2:i] == end for end in sentence_endings):
            last_boundary = i
            break

    # If no sentence boundary found, try to break at the last complete word
    if last_boundary == -1:
        last_space = text[:max_length].rfind(' ')
        if last_space != -1:
            return text[:last_space].rstrip() + "..."
        return text[:max_length].rstrip() + "..."

    return text[:last_boundary].rstrip()


def format_conventional_commit(commit: ConventionalCommit, use_emoji: bool = False, include_description: bool = True,
                               description_length: int = 100) -> str:
    """
    Format a conventional commit message with proper truncation of the description.
    """
    emoji = EMOJI_MAP[commit.type.value] + " " if use_emoji else ""
    scope = f"({commit.scope})" if commit.scope else ""
    formatted_commit = f"{emoji}{commit.type.value}{scope}: {commit.description}"

    if include_description and commit.long_description:
        truncated_description = truncate_to_sentence(commit.long_description, description_length)
        if truncated_description:
            formatted_commit += f"\n\n{truncated_description}"

    return formatted_commit